/*
 * deviced
 *
 * Copyright (c) 2012 - 2013 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef __DEVICE_NOTIFIER_H__
#define __DEVICE_NOTIFIER_H__

enum device_notifier_type {
	DEVICE_NOTIFIER_DAEMON_RESTARTED,
	DEVICE_NOTIFIER_BOOTING_DONE,
	DEVICE_NOTIFIER_LCD,
	DEVICE_NOTIFIER_LCD_OFF,
	DEVICE_NOTIFIER_LCD_OFF_COMPLETE,
	DEVICE_NOTIFIER_LCD_AUTOBRT_SENSING,
	DEVICE_NOTIFIER_LOWBAT,
	DEVICE_NOTIFIER_FULLBAT,
	DEVICE_NOTIFIER_POWER_SUPPLY,
	DEVICE_NOTIFIER_BATTERY_HEALTH,
	DEVICE_NOTIFIER_BATTERY_PRESENT,
	DEVICE_NOTIFIER_BATTERY_OVP,
	DEVICE_NOTIFIER_BATTERY_CHARGING,
	DEVICE_NOTIFIER_DISPLAY_AMBIENT_CONDITION,
	DEVICE_NOTIFIER_DISPLAY_AMBIENT_STATE,
	DEVICE_NOTIFIER_DISPLAY_LOCK,
	DEVICE_NOTIFIER_POWER_RESUME,
	DEVICE_NOTIFIER_POWEROFF,
	DEVICE_NOTIFIER_PROCESS_BACKGROUND,
	DEVICE_NOTIFIER_PROCESS_FOREGROUND,
	DEVICE_NOTIFIER_USB_DEBUG_MODE,
	DEVICE_NOTIFIER_USB_TETHERING_MODE,
	DEVICE_NOTIFIER_EVENT_HANDLER,
	DEVICE_NOTIFIER_CPU_BOOST_LOWBAT,
	DEVICE_NOTIFIER_CPU_BOOST_POWEROFF,
	/* Experimental for Specific device - contact to deviced owner */
	DEVICE_NOTIFIER_EARLY_BOOTING_DONE,
	DEVICE_NOTIFIER_PMQOS,
	DEVICE_NOTIFIER_PMQOS_ULTRAPOWERSAVING,
	DEVICE_NOTIFIER_PMQOS_POWERSAVING,
	DEVICE_NOTIFIER_COOL_DOWN,
	DEVICE_NOTIFIER_VITAL_STATE,
	DEVICE_NOTIFIER_LONGKEY_RESTORE,
	DEVICE_NOTIFIER_UPSM,
	DEVICE_NOTIFIER_UPSM_OFF,
	DEVICE_NOTIFIER_BEZEL_WAKEUP,
	DEVICE_NOTIFIER_DISPLAY_BRIGHTNESS,
	DEVICE_NOTIFIER_ULTRAPOWERSAVING,
	DEVICE_NOTIFIER_CRITICAL_LOG,
	DEVICE_NOTIFIER_EXTCON_COUNT,
	DEVICE_NOTIFIER_MAX,
};

typedef enum _device_notifier_state {
	DEVICE_NOTIFIER_STATE_STOP,
	DEVICE_NOTIFIER_STATE_START,
	DEVICE_NOTIFIER_STATE_CHANGED,
	/* add more states here */
} device_notifier_state_e;

typedef int (*notify_cb)(void *data);
/*
 * This is for internal callback method.
 */
int __register_notifier(enum device_notifier_type type, notify_cb func, const char *caller);
#define register_notifier(type, func) __register_notifier(type, func, __func__)
int __unregister_notifier(enum device_notifier_type type, notify_cb func, const char *caller);
#define unregister_notifier(type, func) __unregister_notifier(type, func, __func__)
void device_notify(enum device_notifier_type type, void *value);
void device_notify_once(enum device_notifier_type status, void *data);

#endif /* __DEVICE_NOTIFIER_H__ */
